
------------------------------------------------------------------------------------
-- Name : nUI_InfoPanel_DecursiveRecount                                          
-- Copyright : Tina Kirby AKA Xrystal (C) 2011 All Rights Reserved      
-- Contact : xrystal@swangen.co.uk                                           
-- Download Sites :                                                          
-- http://www.wowinterface.com/downloads/... (Watch this space)
-- Versions :
-- 1.01.00 - 30th June 2011 - New Addon ( 4.2.0 )
------------------------------------------------------------------------------------

--[[ Use Addon Wide Data Table ]]--
local addonName,addonData = ...

--[[ Localise Addon Wide Data Table Sub Tables ]]--
local Translate = addonData.Translate

--[[ Make sure we have nUI tables ]]--
if not nUI_InfoPanels then nUI_InfoPanels = {}; end

--[[ Set Up nUI InfoPanel Values ]]--
nUI_INFOMODE_DECURSIVERECOUNT  = 16;
nUI_INFOPANEL_DECURSIVERECOUNT = "nUI_InfoPanel_DecursiveRecount";

--[[ Some Constants for Decursive ]]--
MAX_PER_LINE = 8;
MAX_TOTAL  = 64;
MIN_SCALE = 0.65;

--[[ Variables to Store The Existing Values ]]--
local OrigDebuffsFrameMaxCount;
local OrigDebuffsFramePerline;
local OrigDebuffsFrameElemScale;

--[[ Set Up the Info Panel Table ]]--
nUI_InfoPanels[nUI_INFOPANEL_DECURSIVERECOUNT ] =
{	
	enabled   = true,
	desc      = Translate["INFO_PANEL_TEXT"],		-- player friendly name/description of the panel
	label     = Translate["INFO_PANEL_LABEL"],		-- label to use on the panel selection button face
	rotation  = nUI_INFOMODE_DECURSIVERECOUNT,		-- index or position this panel appears on/in when clicking the selector button
	full_size = true, 								-- this plugin requires the entire info panel port without the button bag
	
	options  =
	{
		enabled  = true,
	},
};

--[[ Create the InfoPanel Plugin ]]--
local plugin    = CreateFrame( "Frame", nUI_INFOPANEL_DECURSIVERECOUNT, nUI_Dashboard.Anchor );
plugin.active   = true;

--[[ Handle the addons Events ]]--
local function onEvent(self,event,arg1,arg2,arg3)

	if ( event == "ADDON_LOADED" ) then
		
		if ( arg1 == "Decursive" or arg1 == "Recount" or arg1 == "nUI_InfoPanel_Decursive" or arg1 == addonName ) then print(arg1," has Loaded") end
		if ( arg1 ~= addonName ) then return end

		-- If the addons exist make sure they are loaded
		if not IsAddOnLoaded( "Decursive" ) then 
			LoadAddOn( "Decursive" );
		end
		if not IsAddOnLoaded( "Recount" ) then 
			LoadAddOn( "Recount" );
		end
		if not IsAddOnLoaded( "nUI_InfoPanel_Decursive") then
			LoadAddOn("nUI_InfoPanel_Decursive");
		end
		
		-- Disable the built in Recount plugin	
		if ( nUI_InfoPanels[nUI_INFOPANEL_RECOUNT] ) then
			nUI_InfoPanels[nUI_INFOPANEL_RECOUNT].enabled = false;
		end
		
		-- Disable the Decursive plugin if it exists
		if ( nUI_InfoPanels[nUI_INFOPANEL_DECURSIVE] ) then
			nUI_InfoPanels[nUI_INFOPANEL_DECURSIVE].enabled = false;
		end
		
		-- If both required addons are loaded and available then activate the plugin
		if ( IsAddOnLoaded( "Decursive" ) and IsAddOnLoaded( "Recount" ) ) then 
			plugin.active = true;
		else
			plugin.active = false;
			
			-- If Recount is loaded and available then use the built in plugin
			if ( IsAddOnLoaded( "Recount" ) ) then
				nUI_InfoPanels[nUI_INFOPANEL_RECOUNT].enabled = true;
			end
			
			-- If Decursive is loaded and available then use its plugin if available
			if ( IsAddOnLoaded( "nUI_InfoPanel_Decursive") ) then
				nUI_InfoPanels[nUI_INFOPANEL_DECURSIVE].enabled = true;
			end
			
		end
		self:UnregisterEvent(event);
		
	end	
end

--[[ Set Up the Event Script ]]--
plugin:SetScript( "OnEvent", onEvent );
plugin:RegisterEvent( "ADDON_LOADED" );


--[[ Disable the Decursive Frame ]]--
disableDecursive = function()
	local Decursive = plugin.Decursive;
	if Decursive.saved_parent then
		nUI_Movers:lockFrame( Decursive, false, nil );
		Decursive:SetParent( Decursive.saved_parent );
		--Decursive:SetBackdropBorderColor( Decursive.border_color );
		--Decursive:SetBackdropColor( Decursive.backdrop_color );
		
		-- Get information from the Decursive Addon
		local Dcr = DecursiveRootTable.Dcr;   -- Added 4.0.6
		local DecursiveOptions = Dcr.profile;
		local DecursiveFrames = Dcr.MicroUnitF;

		-- Restore original settings if this addon is disabled
		DecursiveOptions.DebuffsFrameMaxCount = OrigDebuffsFrameMaxCount;
		DecursiveOptions.DebuffsFramePerline = OrigDebuffsFramePerline;
		DecursiveOptions.DebuffsFrameElemScale = OrigDebuffsFrameElemScale;
		DecursiveOptions.ShowDebuffsFrame = true;
		DecursiveFrames.Frame:SetScale(DecursiveOptions.DebuffsFrameElemScale);
		DecursiveFrames:ResetAllPositions();
		DecursiveFrames:Delayed_MFsDisplay_Update();
		
	end
end

disableRecount = function()
	local rframe = plugin.rframe;
	
	if rframe.saved_parent then

		nUI_Movers:lockFrame( rframe, false, nil );
		Recount:LockWindows( false );
		
		rframe:SetParent( rframe.saved_parent );		
		rframe:SetBackdropBorderColor( rframe.border_color );
		rframe:SetBackdropColor( rframe.backdrop_color );

		rframe.Show = rframe.cachedShow;
		rframe.Hide = rframe.cachedHide;
		
		rframe:SetAlpha( 1 );
		
	end

end

enableRecount = function()
	local rframe = Recount.MainWindow;
	
	plugin.rframe = rframe;
	plugin.combat = InCombatLockdown();
	
	if not rframe.saved_parent then
		rframe.saved_parent   = rframe:GetParent();
		rframe.border_color   = rframe:GetBackdropBorderColor();
		rframe.backdrop_color = rframe:GetBackdropColor();
	end
	
	rframe:SetParent( plugin.container );
	--rframe:SetPoint( "TOPRIGHT", plugin.container, "TOPRIGHT", 0, 12.5 );
	--rframe:SetPoint( "BOTTOMLEFT", plugin.container, "BOTTOM", 0, 0 );
	rframe:SetPoint( "TOPRIGHT", plugin.container, "TOPRIGHT", -5, -5 );
	rframe:SetPoint( "BOTTOM", plugin.container, "BOTTOM", 0 , 5 );
	rframe:SetFrameStrata( plugin.container:GetFrameStrata() );
	rframe:SetFrameLevel( plugin.container:GetFrameLevel()+1 );
	rframe:SetBackdropBorderColor( 0, 0, 0, 1 );
	rframe:SetBackdropColor( 0, 0, 0, 0 );
	rframe:Show();
	
	rframe.CloseButton:Hide();			

	rframe.cachedShow = rframe.Show;
	rframe.cachedHide = rframe.Hide;
	
	rframe.Show = function() end;
	rframe.Hide = function() end;

	Recount:LockWindows( true );
	nUI_Movers:lockFrame( rframe, true, nil );
end

--[[ Enable the Decursive Frame ]]--
enableDecursive = function()
	local Decursive = DcrMUFsContainer;
	plugin.Decursive = Decursive;
	
	-- Get information from the Decursive Addon
	local Dcr = DecursiveRootTable.Dcr;   -- Added 4.0.6
	local DecursiveOptions = Dcr.profile;
	local DecursiveFrames = Dcr.MicroUnitF;
	
	-- Save some important information
	if not Decursive.saved_parent then
		Decursive.saved_parent   = Decursive:GetParent();
		--Decursive.border_color   = Decursive:GetBackdropBorderColor();
		--Decursive.backdrop_color = Decursive:GetBackdropColor();
		
		-- Store the existing settings
		OrigDebuffsFrameMaxCount = DecursiveOptions.DebuffsFrameMaxCount;
		OrigDebuffsFramePerline = DecursiveOptions.DebuffsFramePerline;
		OrigDebuffsFrameElemScale = DecursiveOptions.DebuffsFrameElemScale;
				
	end
	
	-- Set the Decursive Frame to fit into the info panel
	nUI_Movers:lockFrame( Decursive, false, nil );
	Decursive:SetParent( plugin.container );
	-- Updated 4.1.0
	--Decursive:SetPoint( "TOPLEFT", plugin.container, "TOPLEFT", 10, -10 );	
	--Decursive:SetPoint( "BOTTOM", plugin.container, "BOTTOM", 0, 0 );
	Decursive:SetPoint( "BOTTOMLEFT", plugin.container, "BOTTOMLEFT", 0, 0 );	
	Decursive:SetFrameStrata( plugin.container:GetFrameStrata() );
	Decursive:SetFrameLevel( plugin.container:GetFrameLevel()+1 );
	--Decursive:SetBackdropBorderColor( 0, 0, 0, 0 );
	--Decursive:SetBackdropColor( 0, 0, 0, 0 );
	
	local ContainerWidth = math.floor(plugin.container:GetWidth() / 2) - 10;
	local ContainerHeight = math.floor(plugin.container:GetHeight()) - 10;
	local DecursiveFrameSize = math.floor(Decursive:GetWidth());
	local DecursiveFrameXGap = DecursiveOptions.DebuffsFrameXSpacing;
	local DecursiveFrameYGap = DecursiveOptions.DebuffsFrameYSpacing;
	
	local RequestedPerLine = DecursiveOptions.DebuffsFramePerline;
	local RequestedMaxCount = DecursiveOptions.DebuffsFrameMaxCount;
	local RequestedScale = DecursiveOptions.DebuffsFrameElemScale;
	
	-- Validate these values so that they do not overstep their bounds
	if ( RequestedPerLine > MAX_PER_LINE ) then RequestedPerLine = MAX_PER_LINE; end
	if ( RequestedMaxCount > MAX_TOTAL ) then RequestedMaxCount = MAX_TOTAL; end
	if ( RequestedScale < MIN_SCALE ) then RequestedScale = MIN_SCALE; end
	
	-- Calculate the boxes down and overall width and height from these values
	local RequestedPerCol = math.floor(RequestedMaxCount / RequestedPerLine);
	local RequestedWidth = math.floor((( DecursiveFrameSize + DecursiveFrameXGap ) * RequestedPerLine ) * RequestedScale );
	local RequestedHeight = math.floor((( DecursiveFrameSize + DecursiveFrameYGap ) * RequestedPerCol ) * RequestedScale );
	
	local tryScale = RequestedScale;
	local tryWidth = RequestedWidth;
	local tryHeight = RequestedHeight;
	local tryPerLine = RequestedPerLine;
	local tryPerCol = RequestedPerCol;

	-- If the requested width or height are too big for the panel segment then
	-- scale it down by 0.05 until it either fits or reaches our MIN_SCALE
	while true do
		if ( tryWidth > ContainerWidth or tryHeight > ContainerHeight ) then
			if ( tryScale < MIN_SCALE + 0.05 ) then
				break;
			else
				tryWidth = math.floor(RequestedWidth / RequestedScale);
				tryHeight = math.floor(RequestedHeight / RequestedScale);
				tryScale = tryScale - 0.05;
				tryWidth = math.floor(tryWidth * tryScale);
				tryHeight = math.floor(tryHeight * tryScale);
			end
		else
			break
		end
	end
	
	-- Update the Values currently set to reflect any changes made in the previous loop
	RequestedWidth = tryWidth;
	RequestedHeight = tryHeight;
	RequestedScale = tryScale;

	-- If the width is still too big then cut down the number of boxes across 
	-- until it gets to 1 or fits.
	while true do
		if ( tryWidth > ContainerWidth ) then
			if ( tryPerLine < 2 ) then
				break;
			else
				tryWidth = RequestedWidth / RequestedScale;
				tryWidth = RequestedWidth / RequestedPerLine;
				tryPerLine = tryPerLine - 1;
				tryWidth = tryWidth * tryPerLine;
				tryWidth = tryWidth * RequestedScale;			
			end
		else
			break;
		end
	end
	
	-- Update the current values
	RequestedWidth = tryWidth;
	RequestedPerLine = tryPerLine;
	
	-- If the height is still too big then cut down the number of boxes down 
	-- until it gets to 1 or fits.
	while true do
		if ( tryHeight > ContainerHeight ) then
			if ( tryPerCol < 2 ) then 
				break;
			else
				tryHeight = RequestedHeight / RequestedScale;
				tryHeight = RequestedHeight / RequestedPerCol;
				tryPerCol = tryPerCol - 1;
				tryHeight = tryHeight * tryPerCol;
				tryHeight = tryHeight * tryScale;				
			end
		else
			break;
		end
	end
	
	-- Update the current values
	RequestedHeight = tryHeight;
	RequestedPerCol = tryPerCol;

	-- Set the Decursive Options to reflect the new values
	DecursiveOptions.DebuffsFrameMaxCount = RequestedPerCol * RequestedPerLine;
	DecursiveOptions.DebuffsFramePerline = RequestedPerLine;
	DecursiveOptions.DebuffsFrameElemScale = RequestedScale;
	DecursiveOptions.ShowDebuffsFrame = true;
	DecursiveFrames.Frame:SetScale(DecursiveOptions.DebuffsFrameElemScale);
	DecursiveFrames:ResetAllPositions();
	DecursiveFrames:Delayed_MFsDisplay_Update();
	
	nUI_Movers:lockFrame( Decursive, true, nil );

end

--[[ Resize the Decursive Frame ]]--
resizeDecursive = function(height,width)
	local Decursive   = plugin.Decursive;
	nUI_Movers:lockFrame( Decursive, false, nil );
	Decursive:SetWidth( width / 2 ); 
	Decursive:SetHeight( height ); 
	nUI_Movers:lockFrame( Decursive, true, nil );
end

--[[ Resize the Recount Frame ]]--
resizeRecount = function(height,width)
	local rframe   = plugin.rframe;
	
	plugin.scale = scale;

	nUI_Movers:lockFrame( rframe, false, nil );
	Recount:LockWindows( false );

	rframe:SetWidth( width / 2 ); 
	rframe:SetHeight( height+10 ); 

	Recount:ResizeMainWindow();
	Recount:LockWindows( true );
	nUI_Movers:lockFrame( rframe, true, nil );
end

--[[ The InfoPanel initialisation routine ]]--
plugin.initPanel = function( container, options )
	plugin.container = container;
	plugin.options   = options;
	if options and options.enabled then
		plugin.setEnabled( true );
	end
end

--[[ The InfoPanel resizing routine ]]--
plugin.sizeChanged = function( scale, height, width )
	local options  = plugin.options;
	plugin.scale = scale;
	
	resizeDecursive(height,width);
	resizeRecount(height,width);
end	

--[[ The InfoPanel enabling routine ]]--
plugin.setEnabled = function( enabled )
	if plugin.enabled ~= enabled then
		plugin.enabled = enabled;
		if not enabled then
			disableRecount();
			disableDecursive();
		else
			enableDecursive();
			enableRecount();
		end				
	end			
end

--[[ The InfoPanel selection routine ]]--
plugin.setSelected = function( selected )
	if selected ~= plugin.selected then
		plugin.selected = selected;
		if selected then
		else
		end
	end
end
